<?php namespace Myriad\NotifyClient\Client\Helpers;

use Exception;
use Myriad\NotifyClient\Builders\NotificationBuilder;
use Myriad\NotifyClient\Client\ValidatesAgainstConfig;
use Myriad\NotifyClient\Models\Notification;
use Myriad\NotifyClient\Utilities\BuiltWithConfig;
use Myriad\NotifyClient\Utilities\HydratesModels;

class NotificationHelper {

  use BuiltWithConfig;
  use HydratesModels;
  use ValidatesAgainstConfig;

  protected function getItemClass() {
    return Notification::class;
  }

  public function makeArray($object) {
    if ($object instanceof NotificationBuilder) {
      return $object->build()->toArray();
    }
    if ($object instanceof Notification) {
      return $object->toArray();
    }
    throw new Exception('Object must be a Notification, or NotificationBuilder');
  }

  public function maybeValidateBuilder ($notification) {
    if ($notification instanceof NotificationBuilder) {
      $filterBuilder = $notification->filterBuilder;
      if ($filterBuilder->shouldValidate) {
        if (empty($filterBuilder->build())) {
          throw new \Exception("The filter has no constraints. This means that *all* registered users would
          receive this message -- you probably don't want that. If you do, you can use the 'withoutValidation'
          method on the FilterBuilder to disable this check, or you can use FilterBuilder::raw and construct
          your own mongo filter. ");
        }
        $this->maybeAssertChannelsAreValid($filterBuilder->channels ?: []);
        $this->maybeAssertSubscriptionsAreValid($filterBuilder->subscriptions ?: []);
      }
    }
  }

  public function uri($notificationId = null) {
    return "notifications" . ($notificationId ? "/$notificationId" : '');
  }
}