<?php namespace Myriad\NotifyClient\Client;

use function array_key_exists;
use function func_get_args;
use Myriad\NotifyClient\HttpInterface;

/**
 * Class HttpWrapper
 * This class serves as a facade for interacting with the uri builder and the http client
 * @package Myriad\NotifyClient\Client
 */
class HttpWrapper {

  /**
   * The HTTP client.
   */
  private $http;

  function __construct(Config $config, HttpInterface $http) {
    $this->config = $config;
    $this->http = $http;
  }

  /**
   * @param $verb
   * @param $arguments
   * @return mixed
   * @throws \Exception
   */
  private function makeRequest($verb, $arguments) {
    $arg1 = array_get($arguments, 0);
    $arg2 = array_get($arguments, 1);

    $url = $this->config->getBaseUrl();
    if (is_string($arg1)) {
      $url .= $arg1;
      $json = $arg2;
    } else {
      $json = $arg1;
    }

    $headers = [
      'Accept' => 'application/json',
      'Content-Type' => 'application/json',
      'Notify-Authorization' => 'Notify-Server ' . $this->config->getToken()
    ];

    $response = $this->http->request($verb, $url, $headers, $json);

    if (is_array($response) && array_key_exists('data', $response)) {
      return $response['data'];
    } else {
      return $response;
    }
  }

  public function get () {
    return $this->makeRequest('get', func_get_args());
  }

  public function put () {
    return $this->makeRequest('put', func_get_args());
  }

  public function post () {
    return $this->makeRequest('post', func_get_args());
  }

  public function delete () {
    return $this->makeRequest('delete', func_get_args());
  }
}